/******************************************************************/
/*                                                                */
/* Program: plotcocoon.c, version 1.05                            */
/*                                                                */
/* First version:                October 2016                     */
/* Latest update:                  2018-01-13                     */
/*                                                                */
/* Program by:        Stefan Spaennare, Lund, Sweden              */
/* E-mail:            stefan.sp@outlook.com                       */
/* Home page:         http:/www.spaennare.se                      */
/* Project web page:  http://www.spaennare.se/tmplotter           */
/*                                                                */
/* Description:                                                   */
/*                                                                */
/* This program plots or mills a cocoon curve.                    */
/*                                                                */
/* The program suggests suitable input parameters. First run      */
/* the program without using a pen or milling tool.               */
/*                                                                */
/* Read the readme.txt document as a whole before you start to    */
/* use the program. The program uses the file stepcfg2.txt that   */
/* contains stepper motor aspect ratio information for the curve. */
/* This must be calibrated before starting to use the program.    */
/*                                                                */
/* It is assumed that you have a PCI-DIO024 digital I/O card in   */
/* your computer connected to your stepper motor drives. This     */
/* I/O card use +5 V logic. If your stepper motor drives have     */
/* opto coupler inputs you need a +5 V driver circuit in between. */
/*                                                                */
/* The stepper motor drives are connected to Port A of the        */
/* PCI-DIO24 card with the following pins at the 37-pin D-SUB     */
/* connector.                                                     */
/*                                                                */
/* Pin 19: Signal ground.                                         */
/*                                                                */
/* Pin 37: Stepper motor 1 clock/step.                            */
/* Pin 36: Stepper motor 1 dir cw/ccw.                            */
/* Pin 35: Stepper motor 1 enable (active high).                  */
/*                                                                */
/* Pin 34: Stepper motor 2 clock/step.                            */
/* Pin 33: Stepper motor 2 dir cw/ccw.                            */
/* Pin 32: Stepper motor 2 enable (active high).                  */
/*                                                                */
/******************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/




/* Include files */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <windows.h>
#include ".\cbw.h"


/* Prototypes */

void ClearScreen (void);
void GetTextCursor (int *x, int *y);
void MoveCursor (int x, int y);
BOOL ask();
int getNewBoardNumber(int col, int row);

double timetic=(double)(CLOCKS_PER_SEC);

char cfg_name[21]="stepcfg2.txt";


double harm_delay(long delay0) {

   long j;
   double sum;


      sum=0.0;

      for (j=1; j<=delay0; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      return(sum);

} /* harm_delay */


void main() {

	/* Variable Declarations */

	FILE *cfgfile;

	int Row, Col;
	int BoardNum = 0;
	int ULStat = 0;
	int PortNum, Direction;
	WORD DataValue;
	int BitLevel;
	int BitNum;
	WORD BitValue;
	int hilo;
	float    RevLevel = (float)CURRENTREVNUM;
	char s[32];  // string for user input

	char cc;

	long i, j,k,d0,d1, ri,stepsx,stepsy,dirx,diry;

	double sum, sum0,v,v0,dv,r0,vr,r,v1,mmx,mmy,ccrad1;
	double x1, y1,x2,y2,dx,dy,x0,y0,ex,ey,rr,r1;
	double x7, y7, aex, aey,dr,xfac,yfac;

	double pi1 = 3.14159265358979323;


	/* Declare UL Revision Level */

	ULStat = cbDeclareRevision(&RevLevel);

	/* Initiate error handling

		Parameters:
			PRINTALL :all warnings and errors encountered will be printed
			DONTSTOP :program will continue even if error occurs.
					 Note that STOPALL and STOPFATAL are only effective in
					 Windows applications, not Console applications.
   */

	cbErrHandling(PRINTALL, DONTSTOP);

	/* set up the display screen */

	/*
	ClearScreen();
	*/

	/* specify the port number */

	PortNum = FIRSTPORTA;
	Direction = DIGITALOUT;
	ULStat = cbDConfigPort(BoardNum, PortNum, Direction);


	/* get a user value to write to the port */

	DataValue = 8;


	/* write BitValue to BitNum of FIRSTPORTA

		Parameters:
			BoardNum    :the number used by CB.CFG to describe this board
			PortNum     :must be FIRSTPORTA
			BitNum      :the number of the bit to be set to BitValue
			BitValue    :the value written to the port   */

	for (j = 0; j <= 7; j++) {

	   BitNum = j;
	   BitValue = 0;

	   ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

    } /* for j */

	BitNum = 2;
	BitValue = 1;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	BitNum = 5;
	BitValue = 1;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);


	if ((cfgfile = fopen(cfg_name, "r")) == NULL) {
		printf("File %s not found.\n", cfg_name);
		exit(0);
	} /* if */

	fscanf(cfgfile,"%d %lf %d", &stepsx, &mmx, &dirx);
	fscanf(cfgfile,"%d %lf %d", &stepsy, &mmy, &diry);

	if ((dirx != 0) && (dirx != 1)) {
		printf("\n\ndirx not 0 or 1. Program terminated.\n\n");
		exit(0);
	} /* if */

	if ((diry != 0) && (diry != 1)) {
		printf("\n\ndiry not 0 or 1. Program terminated.\n\n");
		exit(0);
	} /* if */

	printf("\nCocoon Curve\n");
	printf("------------\n\n");
	printf("\nInput Cocoon Curve radius (float 90.0 mm): ");
	scanf("%lf", &ccrad1);
	printf("\nInput stepper motor delay (int 400): ");
	scanf("%d", &d1);
	printf("\n\n");

    printf("Cocoon curve radius:   %9.3lf mm\n", ccrad1);
	printf("Stepper motor delay: %6d delay units\n\n", d1);
	printf("stepsx mmx dirx:     %6d %9.3lf %3d\n", stepsx, mmx, dirx);
	printf("stepsy mmy diry:     %6d %9.3lf %3d\n", stepsy, mmy, diry);

	cc = 1;

	printf("\nStart milling or plotting (Y/n return)? ");

	scanf("%s", &cc);

	if (cc != 'Y') {
		printf("\n\nProgram terminated.\n\n");
		exit(0);
	} /* if */
    
	d0 = 1000;
	sum0 = 0.0;


	r1 = 0.0;
	v1 = 0.0;
	vr = 76.0*pi1;

	xfac = (ccrad1/65.0)*0.0005*(float)(stepsx)/mmx;
	yfac = (ccrad1/65.0)*0.0005*(float)(stepsy)/mmy;

	x0 = xfac*0.0;
	y0 = yfac*0.0;
	x1 = x0;
	y1 = y0;

	k = 0;


    while (v1<vr) {

		if ((k % 100) == 0) {
			printf("%8d %12.5f %12.3f %12.3f %12.3f\n", k, v1, x1, y1, r1);
		} /* if */

		x2 = xfac*r1*cos(v1) + x0;
		y2 = yfac*r1*sin(v1*1.03) + y0;

		dx = x2 - x1;
		dy = y2 - y1;

		rr = sqrt(dx*dx + dy*dy);

		if (rr > 0.0) {
			ex = dx / rr;
			ey = dy / rr;
		}
		else {
			ex = 0.0;
			ey = 0.0;
		} /* if */

		aex = fabs(ex);
		aey = fabs(ey);

		if (dx >= 0.0) {
			BitNum = 1;
			BitValue = dirx;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		}
		else {

			BitNum = 1;
			BitValue = 1-dirx;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		} /* if */

		if (dy >= 0.0) {
			BitNum = 4;
			BitValue = diry;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		}
		else {

			BitNum = 4;
			BitValue = 1-diry;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		} /* if */


		ri = (long)(rr);
		x7 = 0.0;
		y7 = 0.0;

		for (i = 1; i < ri; i++) {

			x7 = x7 + aex;
			y7 = y7 + aey;

			if (x7 > 1.0) {

				x7 = x7-1.0;

				BitNum = 0;
				BitValue = 1;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(d1 * d0);

				BitNum = 0;
				BitValue = 0;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(d1 * d0);

			} /* if */

			if (y7 > 1.0) {

				y7 = y7-1.0;

				BitNum = 3;
				BitValue = 1;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(d1 * d0);

				BitNum = 3;
				BitValue = 0;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(d1 * d0);

			} /* if */


		} /* for i */

		x1 = x1 + (double)(ri)*ex;
		y1 = y1 + (double)(ri)*ey;

		v1 = v1 + 0.01*pi1*0.5;
		r1 = r1 + 22.5*0.025*50.0*1.55*14.0/40.0*0.5;

		k++;


    } /* while */


	printf("%8d %12.5f %12.3f %12.3f %12.3f\n", k, v1, x1, y1, r1);

	BitNum = 2;
	BitValue = 0;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	BitNum = 5;
	BitValue = 0;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	for (j = 0; j <= 7; j++) {

		BitNum = j;
		BitValue = 0;

		ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	} /* for j */


    printf("\n%12.5f\n",sum0);

	fclose(cfgfile);


    } /* End program */





/***************************************************************************
*
* Name:      reportCurrentBoardNumber(int boardNum)
* Arguments: boardNum - boardNumber
* Returns:   nothing
*
* Queries user and returns the selected board number
*
***************************************************************************/
void reportCurrentBoardNumber(int boardNum)
{
    int c,r;
	GetTextCursor(&c,&r);
	MoveCursor(0,2);
	printf("Currently selected board number is %i.   ",boardNum);
	MoveCursor(c,r);
}

/***************************************************************************
*
* Name:      getNewBoardNumber(int col, int row)
* Arguments: screen position: col = column, row
* Returns:   number from 0 to 63
*
* Queries user and returns the selected board number
*
***************************************************************************/
int getNewBoardNumber(int col, int row)
{
	int bdNum = -1;
	char s[32];
	do
	{
        MoveCursor (col, row);
		printf("Enter new board number -->                        ");
        MoveCursor (col+32, row);
		gets(s);
		sscanf(s,"%i",&bdNum);
	}
	while ((bdNum<0) || (bdNum>64));
	return bdNum;
}

/***************************************************************************
*
* Name:      ask(char *question)
* Arguments: char *question
* Returns:   TRUE if user responds y, FALSE otherwise
*
* Queries user for yes or no response
*
***************************************************************************/

BOOL ask(char *question)
{
   BOOL retval = FALSE;
   char response;
   printf(question);
   response = (char)getchar();
   switch (response)
   {
     case 'y' :
     case 'Y' : retval = TRUE;
   }
   printf("\n\n");
   return retval;
}


/***************************************************************************
*
* Name:      ClearScreen
* Arguments: ---
* Returns:   ---
*
* Clears the screen.
*
***************************************************************************/

#define BIOS_VIDEO   0x10

void
ClearScreen (void)
{
	COORD coordOrg = {0, 0};
	DWORD dwWritten = 0;
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	if (INVALID_HANDLE_VALUE != hConsole)
		FillConsoleOutputCharacter(hConsole, ' ', 80 * 50, coordOrg, &dwWritten);

	MoveCursor(0, 0);

    return;
}


/***************************************************************************
*
* Name:      MoveCursor
* Arguments: x,y - screen coordinates of new cursor position
* Returns:   ---
*
* Positions the cursor on screen.
*
***************************************************************************/


void
MoveCursor (int x, int y)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);

	if (INVALID_HANDLE_VALUE != hConsole)
	{
		COORD coordCursor;
		coordCursor.X = (short)x;
		coordCursor.Y = (short)y;
		SetConsoleCursorPosition(hConsole, coordCursor);
	}

    return;
}


/***************************************************************************
*
* Name:      GetTextCursor
* Arguments: x,y - screen coordinates of new cursor position
* Returns:   *x and *y
*
* Returns the current (text) cursor position.
*
***************************************************************************/

void
GetTextCursor (int *x, int *y)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_SCREEN_BUFFER_INFO csbi;

	*x = -1;
	*y = -1;
	if (INVALID_HANDLE_VALUE != hConsole)
	{
		GetConsoleScreenBufferInfo(hConsole, &csbi);
		*x = csbi.dwCursorPosition.X;
		*y = csbi.dwCursorPosition.Y;
	}

    return;
}





