
/******************************************************************/
/*                                                                */
/* Program: plotspiral.c, version 1.05                            */
/*                                                                */
/* First version:                October 2016                     */
/* Latest update:                  2018-01-13                     */
/*                                                                */
/* Program by:        Stefan Spaennare, Lund, Sweden              */
/* E-mail:            stefan.sp@outlook.com                       */
/* Home page:         http:/www.spaennare.se                      */
/* Project web page:  http://www.spaennare.se/tmplotter           */
/*                                                                */
/* Description:                                                   */
/*                                                                */
/* This program plots or mills a (possibly elliptical) spiral.    */
/*                                                                */
/* The program suggests suitable input parameters. First run      */
/* the program without using a pen or milling tool.               */
/*                                                                */
/* Read the readme.txt document as a whole before you start to    */
/* use the program. The program uses the file stepcfg2.txt that   */
/* contains stepper motor aspect ratio information for the curve. */
/* This must be calibrated before starting to use the program.    */
/*                                                                */
/* It is assumed that you have a PCI-DIO024 digital I/O card in   */
/* your computer connected to your stepper motor drives. This     */
/* I/O card use +5 V logic. If your stepper motor drives have     */
/* opto coupler inputs you need a +5 V driver circuit in between. */
/*                                                                */
/* The stepper motor drives are connected to Port A of the        */
/* PCI-DIO24 card with the following pins at the 37-pin D-SUB     */
/* connector.                                                     */
/*                                                                */
/* Pin 19: Signal ground.                                         */
/*                                                                */
/* Pin 37: Stepper motor 1 clock/step.                            */
/* Pin 36: Stepper motor 1 dir cw/ccw.                            */
/* Pin 35: Stepper motor 1 enable (active high).                  */
/*                                                                */
/* Pin 34: Stepper motor 2 clock/step.                            */
/* Pin 33: Stepper motor 2 dir cw/ccw.                            */
/* Pin 32: Stepper motor 2 enable (active high).                  */
/*                                                                */
/******************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/


/* Include files */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <windows.h>
#include ".\cbw.h"

/* Prototypes */

void ClearScreen (void);
void GetTextCursor (int *x, int *y);
void MoveCursor (int x, int y);
BOOL ask();
int getNewBoardNumber(int col, int row);

double timetic=(double)(CLOCKS_PER_SEC);


char cfg_name[21]="stepcfg2.txt";

double julian_day(year, month, day)
int year, month, day;
{
	double f, g, a, jdd;

	if (month >= 3) {
		f = (double)(year);
		g = (double)(month);
	}
	else {
		f = (double)(year - 1);
		g = (double)(month + 12);
	} /* if */

	a = 2.0 - floor(f / 100.0) + floor(f / 400.0);

	jdd = floor(365.25*f) + floor(30.6001*(g + 1.0)) +
		(double)(day)+a + 1720994.5;

	return(jdd);

} /* julian_day */


double get_float_days(ttime)
time_t *ttime;
{
	int year, month, day, hour, min, sec;

	double fdays, fhours;

	struct tm *tt;

	tt = localtime(ttime);

	year = tt->tm_year + 1900;
	month = tt->tm_mon + 1;
	day = tt->tm_mday;

	hour = tt->tm_hour;
	min = tt->tm_min;
	sec = tt->tm_sec;

	fhours = (double)(hour)+(double)(min) / 60.0 + (double)(sec) / 3600.0;

	fdays = julian_day(year, month, day) + fhours / 24.0;

	return(fdays);

} /* get_float_days */


void day_and_time(ttime)
time_t *ttime;
{

	int year, month, day, hour, min, sec;

	struct tm *tt;

	tt = localtime(ttime);

	year = tt->tm_year + 1900;
	month = tt->tm_mon + 1;
	day = tt->tm_mday;

	hour = tt->tm_hour;
	min = tt->tm_min;
	sec = tt->tm_sec;

	printf("%04d-%02d-%02d %02d:%02d:%02d", year, month, day, hour, min, sec);

} /* day_and_time */


void fday_and_time(outfile, ttime)
FILE *outfile;
time_t *ttime;
{

	int year, month, day, hour, min, sec;

	struct tm *tt;

	tt = localtime(ttime);

	year = tt->tm_year + 1900;
	month = tt->tm_mon + 1;
	day = tt->tm_mday;

	hour = tt->tm_hour;
	min = tt->tm_min;
	sec = tt->tm_sec;

	fprintf(outfile, "%04d-%02d-%02d %02d:%02d:%02d", year, month, day, hour, min, sec);

} /* fday_and_time */


double harm_delay(long delay0) {

   long j;
   double sum;


      sum=0.0;

      for (j=1; j<=delay0; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      return(sum);

} /* harm_delay */


void main() {

	/* Variable Declarations */

	FILE *cfgfile;

	int Row, Col;
	int BoardNum = 0;
	int ULStat = 0;
	int PortNum, Direction;
	WORD DataValue;
	int BitLevel;
	int BitNum;
	WORD BitValue;
	int hilo;
	float    RevLevel = (float)CURRENTREVNUM;
	char s[32];  // string for user input

	char cc;

	long i, j, k, delay0, delay1, delay2, d1, d0, ri;
	long stepsx, stepsy, nrevs1, hour1, min1, dirx, diry;

	double sum, sum0,v,dv,r0x,r0y,mmx,mmy,spiralradiusx,spiralradiusy,revs1;
	double x1, y1,x2,y2,dx,dy,x0,y0,ex,ey,rr,r1x,r1y;
	double x7, y7, aex, aey, drx,dry, xfac, yfac, xyf0;
	double fdays1, fdays2, etime1, sec1;

	double pi1 = 3.14159265358979323;

	time_t start_time, stop_time;


	/* Declare UL Revision Level */

	ULStat = cbDeclareRevision(&RevLevel);

	/* Initiate error handling

		Parameters:
			PRINTALL :all warnings and errors encountered will be printed
			DONTSTOP :program will continue even if error occurs.
					 Note that STOPALL and STOPFATAL are only effective in
					 Windows applications, not Console applications.
   */

	cbErrHandling(PRINTALL, DONTSTOP);

	/* set up the display screen */

	/*
	ClearScreen();
	*/

	/* specify the port number */

	PortNum = FIRSTPORTA;
	Direction = DIGITALOUT;
	ULStat = cbDConfigPort(BoardNum, PortNum, Direction);


	/* get a user value to write to the port */

	DataValue = 8;


	/* write BitValue to BitNum of FIRSTPORTA

		Parameters:
			BoardNum    :the number used by CB.CFG to describe this board
			PortNum     :must be FIRSTPORTA
			BitNum      :the number of the bit to be set to BitValue
			BitValue    :the value written to the port   */

	for (j = 0; j <= 7; j++) {

	   BitNum = j;
	   BitValue = 0;

	   ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

    } /* for j */

	BitNum = 2;
	BitValue = 1;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	BitNum = 5;
	BitValue = 1;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	if ((cfgfile = fopen(cfg_name, "r")) == NULL) {
		printf("File %s not found.\n", cfg_name);
		exit(0);
	} /* if */

	fscanf(cfgfile,"%d %lf %d", &stepsx, &mmx, &dirx);
	fscanf(cfgfile,"%d %lf %d", &stepsy, &mmy, &diry);

	if ((dirx != 0) && (dirx != 1)) {
		printf("\n\ndirx not 0 or 1. Program terminated.\n\n");
		exit(0);
	} /* if */

	if ((diry != 0) && (diry != 1)) {
		printf("\n\ndiry not 0 or 1. Program terminated.\n\n");
		exit(0);
	} /* if */


    printf("\nSpiral Curve\n");
	printf("------------\n\n");
	printf("Input max spiral radius x (float mm): ");
	scanf("%lf", &spiralradiusx);
	printf("Input max spiral radius y (float mm): ");
	scanf("%lf", &spiralradiusy);
	printf("\nInput number of spiral revolutions (float): ");
	scanf("%lf", &revs1);
	printf("\nInput number of straight lines per revolution (int 200): ");
    scanf("%d", &nrevs1);
    printf("\nInput stepper motor delay (int 400): ");
	scanf("%d", &delay0);
	printf("\n\n");

	printf("Spiral radius x:       %9.3lf mm\n", spiralradiusx);
	printf("Spiral radius y:       %9.3lf mm\n", spiralradiusy);
	printf("Spiral revolutions:  %9.3lf\n", revs1);
	printf("Lines per rev:       %6d\n", nrevs1);
	printf("Stepper motor delay: %6d delay units\n\n", delay0);
	printf("stepsx mmx dirx:     %6d %9.3lf %3d\n", stepsx, mmx, dirx);
	printf("stepsy mmy diry:     %6d %9.3lf %3d\n", stepsy, mmy, diry);

	cc = 1;

	printf("\nStart milling or plotting (Y/n return)? ");

	scanf("%s", &cc);

	if (cc != 'Y') {
		printf("\n\nProgram terminated.\n\n");
		exit(0);
	} /* if */

	d0 = 1000;
	sum0 = 0.0;

	v = 0.0;
	dv = 2.0*pi1/(double)(nrevs1);
	r0x = spiralradiusx;
	r0y = spiralradiusy;
	r1x = 0.0;
	r1y = 0.0;
	drx = spiralradiusx/(revs1*(double)(nrevs1));
	dry = spiralradiusy/(revs1*(double)(nrevs1));

	xfac = (double)(stepsx)/mmx;
	yfac = (double)(stepsy)/mmy;

	x0 = xfac*0.0;
	y0 = yfac*0.0;

	x1 = xfac*r1x*cos(v) + x0;
	y1 = yfac*r1y*sin(v) + y0;

	xyf0 = xfac / yfac;

	delay1 = delay0;
	delay2 = delay0;

	if (xyf0 >= 1.0) {
		delay2 = (int)(xyf0*(double)(delay1)+0.5);
	} /* if */

	k = 0;

	time(&start_time);
	printf("Start time: ");
	day_and_time(&start_time);

	printf("\n\n");


    while (r1x < r0x) {

		if ((k % 20) == 0) {
			printf("%6d v: %9.5f x: %9.3f y: %9.3f rx: %9.3f ry: %9.3f\n", k, v, x1, y1, r1x,r1y);
		} /* if */

		k++;

		r1x = r1x + drx;
		r1y = r1y + dry;

		v = v + dv;

		x2 = xfac*r1x*cos(v) + x0;
		y2 = yfac*r1y*sin(v) + y0;

		dx = x2 - x1;
		dy = y2 - y1;

		rr = sqrt(dx*dx + dy*dy);

		if (rr > 0.0) {
			ex = dx / rr;
			ey = dy / rr;
		}
		else {
			ex = 0.0;
			ey = 0.0;
		} /* if */

		aex = fabs(ex);
		aey = fabs(ey);

		if (dx >= 0.0) {
			BitNum = 1;
			BitValue = dirx;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		}
		else {

			BitNum = 1;
			BitValue = 1-dirx;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		} /* if */

		if (dy >= 0.0) {
			BitNum = 4;
			BitValue = diry;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		}
		else {

			BitNum = 4;
			BitValue = 1-diry;

			ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);
		} /* if */


		ri = (long)(rr);
		x7 = 0.0;
		y7 = 0.0;

		for (i = 1; i < ri; i++) {

			x7 = x7 + aex;
			y7 = y7 + aey;

			if (x7 > 1.0) {

				x7 = x7-1.0;

				BitNum = 0;
				BitValue = 1;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(delay1 * d0);

				BitNum = 0;
				BitValue = 0;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(delay1 * d0);

			} /* if */

			if (y7 > 1.0) {

				y7 = y7-1.0;

				BitNum = 3;
				BitValue = 1;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(delay2 * d0);

				BitNum = 3;
				BitValue = 0;

				ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

				sum0 = sum0 + harm_delay(delay2 * d0);

			} /* if */


		} /* for i */

		x1 = x1 + (double)(ri)*ex;
		y1 = y1 + (double)(ri)*ey;


    } /* while */


	printf("%5d v: %9.5f x: %9.3f y: %9.3f rx: %9.3f ry: %9.3f\n\n", k, v, x1, y1, r1x,r1y);

	time(&stop_time);
	printf("Stop time:  ");
	day_and_time(&stop_time);

	fdays1 = get_float_days(&start_time);
	fdays2 = get_float_days(&stop_time);

	etime1 = fdays2 - fdays1;
	hour1 = (int)(etime1 * 24.0);
	min1 = (int)(24.0*60.0*etime1-60.0*(double)(hour1));
	sec1 = (24.0*3600.0*etime1-3600.0*(double)(hour1)-60.0*(double)(min1));

	printf("\n\nElapsed clock time: %6d h %6d m %8.3f s ; %12.3f tot seconds\n", hour1, min1, sec1, 24.0*3600.0*etime1);

    BitNum = 2;
	BitValue = 0;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	BitNum = 5;
	BitValue = 0;

	ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	for (j = 0; j <= 7; j++) {

		BitNum = j;
		BitValue = 0;

		ULStat = cbDBitOut(BoardNum, PortNum, BitNum, BitValue);

	} /* for j */

    printf("\n%12.5f\n",sum0);

	fclose(cfgfile);

   } /* End program */





/***************************************************************************
*
* Name:      reportCurrentBoardNumber(int boardNum)
* Arguments: boardNum - boardNumber
* Returns:   nothing
*
* Queries user and returns the selected board number
*
***************************************************************************/
void reportCurrentBoardNumber(int boardNum)
{
    int c,r;
	GetTextCursor(&c,&r);
	MoveCursor(0,2);
	printf("Currently selected board number is %i.   ",boardNum);
	MoveCursor(c,r);
}

/***************************************************************************
*
* Name:      getNewBoardNumber(int col, int row)
* Arguments: screen position: col = column, row
* Returns:   number from 0 to 63
*
* Queries user and returns the selected board number
*
***************************************************************************/
int getNewBoardNumber(int col, int row)
{
	int bdNum = -1;
	char s[32];
	do
	{
        MoveCursor (col, row);
		printf("Enter new board number -->                        ");
        MoveCursor (col+32, row);
		gets(s);
		sscanf(s,"%i",&bdNum);
	}
	while ((bdNum<0) || (bdNum>64));
	return bdNum;
}

/***************************************************************************
*
* Name:      ask(char *question)
* Arguments: char *question
* Returns:   TRUE if user responds y, FALSE otherwise
*
* Queries user for yes or no response
*
***************************************************************************/

BOOL ask(char *question)
{
   BOOL retval = FALSE;
   char response;
   printf(question);
   response = (char)getchar();
   switch (response)
   {
     case 'y' :
     case 'Y' : retval = TRUE;
   }
   printf("\n\n");
   return retval;
}


/***************************************************************************
*
* Name:      ClearScreen
* Arguments: ---
* Returns:   ---
*
* Clears the screen.
*
***************************************************************************/

#define BIOS_VIDEO   0x10

void
ClearScreen (void)
{
	COORD coordOrg = {0, 0};
	DWORD dwWritten = 0;
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	if (INVALID_HANDLE_VALUE != hConsole)
		FillConsoleOutputCharacter(hConsole, ' ', 80 * 50, coordOrg, &dwWritten);

	MoveCursor(0, 0);

    return;
}


/***************************************************************************
*
* Name:      MoveCursor
* Arguments: x,y - screen coordinates of new cursor position
* Returns:   ---
*
* Positions the cursor on screen.
*
***************************************************************************/


void
MoveCursor (int x, int y)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);

	if (INVALID_HANDLE_VALUE != hConsole)
	{
		COORD coordCursor;
		coordCursor.X = (short)x;
		coordCursor.Y = (short)y;
		SetConsoleCursorPosition(hConsole, coordCursor);
	}

    return;
}


/***************************************************************************
*
* Name:      GetTextCursor
* Arguments: x,y - screen coordinates of new cursor position
* Returns:   *x and *y
*
* Returns the current (text) cursor position.
*
***************************************************************************/

void
GetTextCursor (int *x, int *y)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_SCREEN_BUFFER_INFO csbi;

	*x = -1;
	*y = -1;
	if (INVALID_HANDLE_VALUE != hConsole)
	{
		GetConsoleScreenBufferInfo(hConsole, &csbi);
		*x = csbi.dwCursorPosition.X;
		*y = csbi.dwCursorPosition.Y;
	}

    return;
}





